<?php
/**
 * account.php
 *
 * Account management
 *
 * @author     Makai Lajos
 * @copyright  2014
 * @license    License.txt
 * @version    2.0
 * @link       http://opencart-api.com/product/opencart-restful-api-pro-v2-0/
 * @see        http://webshop.opencart-api.com/schema_v2.0/
 */
class ControllerExtensionRestAccount extends Controller {

	private $error = array();

	public function getAccount() {

		$json = array('success' => true);

		$this->language->load('account/edit');


		if (!$this->customer->isLogged()) {
			$json["error"] = "User is not logged in";
			$json["success"] = false;
		}
		
		if($json["success"]){
		
			$this->load->model('account/customer');
			
			$user = $this->model_account_customer->getCustomer($this->customer->getId());

			// Custom Fields
			$this->load->model('account/custom_field');

			$user['custom_fields'] = $this->model_account_custom_field->getCustomFields($this->config->get('config_customer_group_id'));

			$user['account_custom_field'] = unserialize($user['custom_field']);

			unset($user["password"]);
			unset($user["salt"]);
			unset($user['custom_field']);			
			$json["data"] = $user;
		}

		if ($this->debugIt) {
			echo '<pre>';
			print_r($json);
			echo '</pre>';
		} else {
			$this->response->setOutput(json_encode($json));
		}
	}

	public function saveAccount($post) {

		$json = array('success' => true);
		
		if (!$this->customer->isLogged()) {
			$json["error"] = "User is not logged in";
			$json["success"] = false;
		}else {
			if ($this->validate($post)) {
				$this->load->model('account/customer');
				$this->model_account_customer->editCustomer($post);			
			}else {
				$json["error"] = $this->error;
				$json["success"] = false;
			}
		}

		if ($this->debugIt) {
			echo '<pre>';
			print_r($json);
			echo '</pre>';
		} else {
			$this->response->setOutput(json_encode($json));
		}					
	}

	protected function validate($post) {
		
		$this->load->model('account/customer');
		$this->language->load('account/edit');

		if(isset($post['firstname'])){
			if ((utf8_strlen($post['firstname']) < 1) || (utf8_strlen($post['firstname']) > 32)) {
				$this->error['firstname'] = $this->language->get('error_firstname');
			}
		}else{
				$this->error['firstname'] = $this->language->get('error_firstname');
		}

		if(isset($post['lastname'])){		
			if ((utf8_strlen($post['lastname']) < 1) || (utf8_strlen($post['lastname']) > 32)) {
				$this->error['lastname'] = $this->language->get('error_lastname');
			}
		}else{
				$this->error['lastname'] = $this->language->get('error_lastname');
		}

		if(isset($post['email'])){
			if ((utf8_strlen($post['email']) > 96) || !preg_match('/^[^\@]+@.*\.[a-z]{2,6}$/i', $post['email'])) {
				$this->error['email'] = $this->language->get('error_email');
			}
		}else{
				$this->error['email'] = $this->language->get('error_email');
		}
		
		if(isset($post['email'])){
			if (($this->customer->getEmail() != $post['email']) && $this->model_account_customer->getTotalCustomersByEmail($post['email'])) {
				$this->error['warning'] = $this->language->get('error_exists');
			}
		}else{
				$this->error['email'] = "E-mail is required";
		}
		
		if(isset($post['telephone'])){
		
			if ((utf8_strlen($post['telephone']) < 3) || (utf8_strlen($post['telephone']) > 32)) {
				$this->error['telephone'] = $this->language->get('error_telephone');
			}
		}else{
				$this->error['telephone'] = $this->language->get('error_telephone');
		}
		
		
		// Custom field validation
		$this->load->model('account/custom_field');

		$custom_fields = $this->model_account_custom_field->getCustomFields($this->config->get('config_customer_group_id'));

		foreach ($custom_fields as $custom_field) {
			if (($custom_field['location'] == 'account') && $custom_field['required'] && empty($post['custom_field'][$custom_field['custom_field_id']])) {
				$this->error['custom_field'][$custom_field['custom_field_id']] = sprintf($this->language->get('error_custom_field'), $custom_field['name']);
			}
		}
		if (!$this->error) {
			return true;
		} else {
			return false;
		}
	}

	/*
	* ACCOUNT FUNCTIONS
	*/	
	public function account() {

		$this->checkPlugin();
		
		if ( $_SERVER['REQUEST_METHOD'] === 'GET' ){
			//get account details

			$this->getAccount();			
		}else if ( $_SERVER['REQUEST_METHOD'] === 'POST' ){
			//modify account
			$requestjson = file_get_contents('php://input');
		
			$requestjson = json_decode($requestjson, true);           

			if (!empty($requestjson)) {       
				$this->saveAccount($requestjson);
			}else {
				$this->response->setOutput(json_encode(array('success' => false)));
			}   

		}

    }


	public function customfield() {
		$this->checkPlugin();

		if ( $_SERVER['REQUEST_METHOD'] === 'GET' ){
			
		$this->load->model('account/custom_field');

		// Customer Group
		if (isset($this->request->get['customer_group_id']) && is_array($this->config->get('config_customer_group_display')) && in_array($this->request->get['customer_group_id'], $this->config->get('config_customer_group_display'))) {
			$customer_group_id = $this->request->get['customer_group_id'];
		} else {
			$customer_group_id = $this->config->get('config_customer_group_id');
		}


		$json['custom_fields'] = $this->model_account_custom_field->getCustomFields($customer_group_id);

		/*$custom_fields = $this->model_account_custom_field->getCustomFields($customer_group_id);

		foreach ($custom_fields as $custom_field) {
			$json[] = array(
				'custom_field_id' => $custom_field['custom_field_id'],
				'required'        => $custom_field['required']
			);
		}*/

	}else {
		$json["error"]		= "Only GET request method allowed";
		$json["success"]	= false;

	}    
		if ($this->debugIt) {
			echo '<pre>';
			print_r($json);
			echo '</pre>';
		} else {
			$this->response->setOutput(json_encode($json));
		}
	}

	private function checkPlugin() {
		
		$this->response->addHeader('Content-Type: application/json');
		
		$json = array("success"=>false);

		/*check rest api is enabled*/
		if (!$this->config->get('rest_api_status')) {
			$json["error"] = 'API is disabled. Enable it!';
		}
	

		$headers = apache_request_headers();
		
		$key = "";

		if(isset($headers['X-Oc-Merchant-Id'])){
			$key = $headers['X-Oc-Merchant-Id'];
		}else if(isset($headers['X-OC-MERCHANT-ID'])) {
			$key = $headers['X-OC-MERCHANT-ID'];
		}
			
		/*validate api security key*/
		if ($this->config->get('rest_api_key') && ($key != $this->config->get('rest_api_key'))) {
			$json["error"] = 'Invalid secret key';
		}

		if(isset($json["error"])){			
			echo(json_encode($json));
			exit;
		}else {
			$this->response->setOutput(json_encode($json));			
		}	
	}

}

if( !function_exists('apache_request_headers') ) {
    function apache_request_headers() {
        $arh = array();
        $rx_http = '/\AHTTP_/';

        foreach($_SERVER as $key => $val) {
            if( preg_match($rx_http, $key) ) {
                $arh_key = preg_replace($rx_http, '', $key);
                $rx_matches = array();
           // do some nasty string manipulations to restore the original letter case
           // this should work in most cases
                $rx_matches = explode('_', $arh_key);

                if( count($rx_matches) > 0 and strlen($arh_key) > 2 ) {
                    foreach($rx_matches as $ak_key => $ak_val) {
                        $rx_matches[$ak_key] = ucfirst($ak_val);
                    }

                    $arh_key = implode('-', $rx_matches);
                }

                $arh[$arh_key] = $val;
            }
        }
        
        return( $arh );
    }
}
?>
