<?php  

/**
 * login.php
 *
 * Login management
 *
 * @author     Makai Lajos
 * @copyright  2014
 * @license    License.txt
 * @version    2.0
 * @link       http://opencart-api.com/product/opencart-restful-api-pro-v2-0/
 * @see        http://webshop.opencart-api.com/schema_v2.0/
 */
class ControllerExtensionRestLogin extends Controller { 
	
	/*
	* Login user
	*/
	public function login() {

		$this->checkPlugin();
		
		$json = array('success' => true);
		
		if ( $_SERVER['REQUEST_METHOD'] === 'POST' ){
			
			$requestjson = file_get_contents('php://input');
		
			$requestjson = json_decode($requestjson, true);

			$post		 = $requestjson;

			$this->language->load('checkout/checkout');
				
			if ($this->customer->isLogged()) {
				$json['error']		= "User already is logged";			
				$json['success']	= false;			
			}	
			
			if ($json['success']) {
				if (!$this->customer->login($post['email'], $post['password'])) {
					$json['error']['warning'] = $this->language->get('error_login');
					$json['success']	= false;
				}
			
				$this->load->model('account/customer');
			
				$customer_info = $this->model_account_customer->getCustomerByEmail($post['email']);
				
				if ($customer_info && !$customer_info['approved']) {
					$json['error']['warning'] = $this->language->get('error_approved');
					$json['success']	= false;
				}		
			}
			
			if ($json['success']) {
				unset($this->session->data['guest']);
					
				// Default Addresses
				$this->load->model('account/address');
					
				if ($this->config->get('config_tax_customer') == 'payment') {
					$this->session->data['payment_address'] = $this->model_account_address->getAddress($this->customer->getAddressId());
				}	

				if ($this->config->get('config_tax_customer') == 'shipping') {
					$this->session->data['shipping_address'] = $this->model_account_address->getAddress($this->customer->getAddressId());
				}

				// Add to activity log
				$this->load->model('account/activity');

				$activity_data = array(
					'customer_id' => $this->customer->getId(),
					'name'        => $this->customer->getFirstName() . ' ' . $this->customer->getLastName()
				);

				$this->model_account_activity->addActivity('login', $activity_data);				

				unset($customer_info['password']);
				$customer_info["session"] = session_id();
				
				// Custom Fields
				$this->load->model('account/custom_field');

				$customer_info['custom_fields'] = $this->model_account_custom_field->getCustomFields($this->config->get('config_customer_group_id'));

				$customer_info['account_custom_field'] = unserialize($customer_info['custom_field']);
				unset($customer_info['custom_field']);

				$json['data'] = $customer_info;
			}
						
		}else {
				$json["error"]		= "Only POST request method allowed";
				$json["success"]	= false;
		}

		if ($this->debugIt) {
			echo '<pre>';
			print_r($json);
			echo '</pre>';
		} else {
			$this->response->setOutput(json_encode($json));
		}
	
	}

	
	private function checkPlugin() {
		
		$this->response->addHeader('Content-Type: application/json');
		
		$json = array("success"=>false);

		/*check rest api is enabled*/
		if (!$this->config->get('rest_api_status')) {
			$json["error"] = 'API is disabled. Enable it!';
		}
	

		$headers = apache_request_headers();
		
		$key = "";

		if(isset($headers['X-Oc-Merchant-Id'])){
			$key = $headers['X-Oc-Merchant-Id'];
		}else if(isset($headers['X-OC-MERCHANT-ID'])) {
			$key = $headers['X-OC-MERCHANT-ID'];
		}
			
		/*validate api security key*/
		if ($this->config->get('rest_api_key') && ($key != $this->config->get('rest_api_key'))) {
			$json["error"] = 'Invalid secret key';
		}

		if(isset($json["error"])){			
			echo(json_encode($json));
			exit;
		}else {
			$this->response->setOutput(json_encode($json));			
		}	
	}


	/*
	Only for testing
	Depracted
	http:/opencart3.my/api/rest/checkuser/key/123
	*/	
	public function checkuser() {

		$json = array('success' => true);	

		if (!$this->customer->isLogged()) {
				$json['error']		= "User is not logged";			
				$json['success']	= false;			
		}else {
			$json['data'] 		= $this->customer->getEmail();			
		}
		
		if ($this->debugIt) {
				echo '<pre>';
				print_r($json);
		} else {
				$this->response->setOutput(json_encode($json));
		}
    }
}
if( !function_exists('apache_request_headers') ) {
    function apache_request_headers() {
        $arh = array();
        $rx_http = '/\AHTTP_/';

        foreach($_SERVER as $key => $val) {
            if( preg_match($rx_http, $key) ) {
                $arh_key = preg_replace($rx_http, '', $key);
                $rx_matches = array();
           // do some nasty string manipulations to restore the original letter case
           // this should work in most cases
                $rx_matches = explode('_', $arh_key);

                if( count($rx_matches) > 0 and strlen($arh_key) > 2 ) {
                    foreach($rx_matches as $ak_key => $ak_val) {
                        $rx_matches[$ak_key] = ucfirst($ak_val);
                    }

                    $arh_key = implode('-', $rx_matches);
                }

                $arh[$arh_key] = $val;
            }
        }
        
        return( $arh );
    }
}
?>
